@@ -5,7 +5,8 @@ use Mojo::Base 'Mojolicious::Plugin';
 use Plack::Util;
 use Mojo::Message::Request;
 use Mojo::Message::Response;
-our $VERSION = '0.28';
+our $VERSION = '0.32';
+use Scalar::Util 'weaken';
     
     ### ---
     ### register
@@ -13,17 +14,14 @@ our $VERSION = '0.28';
     sub register {
         my ($self, $app, $mws) = @_;
         
-        my $inside_app;
-        
         my $plack_app = sub {
             my $env = shift;
-            my $c = $env->{'mojo.c'};
-            my $tx = $c->tx;
+            my $tx = $env->{'mojo.c'}->tx;
             
             $tx->req(psgi_env_to_mojo_req($env));
             
             if ($env->{'mojo.routed'}) {
-                my $stash = $c->stash;
+                my $stash = $env->{'mojo.c'}->stash;
                 for my $key (grep {$_ =~ qr{^mojo\.}} keys %{$stash}) {
                     delete $stash->{$key};
                 }
@@ -31,9 +29,9 @@ our $VERSION = '0.28';
                 my $sever = $tx->res->headers->header('server');
                 $tx->res(Mojo::Message::Response->new);
                 $tx->res->headers->header('server', $sever);
-                $inside_app->();
+                $env->{'mojo.inside_app'}->();
             } else {
-                $inside_app->();
+                $env->{'mojo.inside_app'}->();
                 $env->{'mojo.routed'} = 1;
             }
             
@@ -53,26 +51,21 @@ our $VERSION = '0.28';
         }
         
         $app->hook('around_dispatch' => sub {
-            ($inside_app, my $c) = @_;
+            my ($next, $c) = @_;
             
             if ($c->tx->req->error) {
-                $inside_app->();
-                return;
-            }
-            
-            my $plack_env = mojo_req_to_psgi_env($c->req);
-            
-            local $plack_env->{'mojo.c'} = $c;
-            
-            local $plack_env->{'psgi.errors'} =
-                Mojolicious::Plugin::PlackMiddleware::_EH->new(sub {
-                    $c->app->log->debug(shift);
-                });
-            
-            $c->tx->res(psgi_res_to_mojo_res($plack_app->($plack_env)));
-            
-            if (! $plack_env->{'mojo.routed'}) {
-                $c->rendered;
+                $next->();
+            } else {
+                my $plack_env = mojo_req_to_psgi_env($c->req);
+                $plack_env->{'mojo.c'} = $c;
+                $plack_env->{'mojo.inside_app'} = $next;
+                $plack_env->{'psgi.errors'} =
+                    Mojolicious::Plugin::PlackMiddleware::_EH->new(sub {
+                        $c->app->log->debug(shift);
+                    });
+                
+                $c->tx->res(psgi_res_to_mojo_res($plack_app->($plack_env)));
+                $c->rendered if (! $plack_env->{'mojo.routed'});
             }
         });
     }
@@ -155,13 +148,14 @@ our $VERSION = '0.28';
         $mojo_res->code($psgi_res->[0]);
         my $headers = $mojo_res->headers;
         while (scalar @{$psgi_res->[1]}) {
-            $headers->header(shift @{$psgi_res->[1]} => shift @{$psgi_res->[1]});
+            $headers->add(shift @{$psgi_res->[1]} => shift @{$psgi_res->[1]});
         }
         
         $headers->remove('Content-Length'); # should be set by mojolicious later
         
         my $asset = $mojo_res->content->asset;
         Plack::Util::foreach($psgi_res->[2], sub {$asset->add_chunk($_[0])});
+        weaken($psgi_res);
         return $mojo_res;
     }
     
@@ -174,10 +168,16 @@ our $VERSION = '0.28';
         my $headers = $mojo_res->content->headers;
         my @headers;
         for my $name (@{$headers->names}) {
-            push @headers, $name => join ', ', map {join ', ', @$_} $headers->header($name);
+            push @headers, $name => $headers->header($name);
         }
         my @body;
         my $offset = 0;
+        
+        # don't know why but this block makes long polling tests to pass
+        if ($mojo_res->content->is_dynamic && $mojo_res->content->{delay}) {
+            $mojo_res->get_body_chunk(0);
+        }
+        
         while (length(my $chunk = $mojo_res->get_body_chunk($offset))) {
             push(@body, $chunk);
             $offset += length $chunk;
@@ -230,6 +230,7 @@ use Mojo::Base -base;
 ### ---
 package Mojolicious::Plugin::PlackMiddleware::_Cond;
 use strict;
+use warnings;
 use parent qw(Plack::Middleware::Conditional);
     
     sub call {
